/**
 * This file was auto generated from scripts/generate-version.sh
 */
import type { PackageJson } from 'type-fest'
export const packageJson: PackageJson = {
  "name": "flash-store",
  "version": "1.3.5",
  "description": "FlashStore is a Key-Value persistent storage with easy to use ES6 Map-like API(both Async and Sync support), powered by LevelDB and TypeScript.",
  "type": "module",
  "exports": {
    ".": {
      "import": "./dist/esm/src/mod.js",
      "require": "./dist/cjs/src/mod.js"
    }
  },
  "typings": "./dist/esm/src/mod.d.ts",
  "engines": {
    "node": ">=14"
  },
  "scripts": {
    "clean": "shx rm -fr dist/*",
    "dist": "npm-run-all clean build dist:commonjs",
    "build": "tsc && tsc -p tsconfig.cjs.json",
    "dist:commonjs": "jq -n \"{ type: \\\"commonjs\\\" }\" > dist/cjs/package.json",
    "pack": "npm pack",
    "doc": "npm run dist && echo '# FlashStore v'$(jq -r .version package.json)' Documentation\n' > docs/index.md && jsdoc2md dist/flash-store.js>> docs/index.md",
    "demo": "ts-node example/demo.ts",
    "lint": "npm-run-all lint:es lint:ts lint:md",
    "lint:md": "markdownlint README.md",
    "lint:ts": "tsc --isolatedModules --noEmit",
    "lint:es": "eslint --ignore-pattern fixtures/ \"src/**/*.ts\" \"tests/**/*.ts\"",
    "test": "npm run lint && npm run test:unit",
    "test:pack": "bash -x scripts/npm-pack-testing.sh",
    "test:unit": "tap --node-arg=--loader=ts-node/esm --node-arg=--no-warnings \"src/**/*.spec.ts\" \"tests/**/*.spec.ts\""
  },
  "repository": {
    "type": "git",
    "url": "git+https://github.com/huan/flash-store.git"
  },
  "keywords": [
    "leveldb",
    "nosql",
    "database"
  ],
  "author": "Huan LI <zixia@zixia.net>",
  "license": "Apache-2.0",
  "bugs": {
    "url": "https://github.com/huan/flash-store/issues"
  },
  "homepage": "https://github.com/huan/flash-store#readme",
  "dependencies": {
    "async-map-like": "^0.2.5",
    "brolog": "^1.12.4",
    "cuid": "^2.1.8",
    "level": "^7.0.0",
    "rimraf": "^3.0.2",
    "state-switch": "^0.14.1"
  },
  "devDependencies": {
    "@chatie/eslint-config": "^0.14.1",
    "@chatie/git-scripts": "^0.6.2",
    "@chatie/semver": "^0.4.7",
    "@chatie/tsconfig": "^0.20.2",
    "@types/app-root-path": "^1.2.4",
    "@types/level": "^6.0.0",
    "@types/level-errors": "^3.0.0",
    "@types/rimraf": "^3.0.1",
    "jsdoc-to-markdown": "^7.0.1",
    "npm-run-all": "^4.1.5",
    "shx": "^0.3.3",
    "source-map-support": "^0.5.19",
    "tstest": "^0.5.16",
    "typescript": "^4.4.3"
  },
  "files": [
    "src/",
    "dist/"
  ],
  "publishConfig": {
    "access": "public",
    "tag": "next"
  },
  "tap": {
    "check-coverage": false
  },
  "git": {
    "scripts": {
      "pre-push": "npx git-scripts-pre-push"
    }
  }
} as any
